library("RMVL")

#
# We find maximums of upper limits and SNRs in a given area for each frequency band
#


Matlas<-mvl_open("O3a_3_atlas.mvl")

parameters<-mvl2R(Matlas$parameters)

Msnr<-Matlas$skymaps[,"snr"]
Mul<-Matlas$skymaps[,"ul"]
Mra<-Matlas$skymaps[,"ra"]
Mdec<-Matlas$skymaps[,"dec"]
Mi<-Matlas$skymaps[,"idx"]


# Find maximums over a circle

# atlas coordinates are in radians
RA<- (pi/12.0)*(17+45/60.0+40.03599/3600.0)
DEC<- (pi/180)* -(29+0/60.0+28.1699/3600.0)
# 1 degree
radius<-(1.0*pi/180.0)

process_band<-function(i, idx) {
	# The amount of data increases quadratically with frequency band
	if(i %% 1000==0)cat("\r", (i*1.0)^2*100.0/(dim(parameters)[1]*1.0)^2, "%          ")

	# Discard any indices from hash collisions. Usually there are none, so idx is unchanged.
	idx<-idx[Mi[idx]==parameters[i, "idx"]]

	# Enlarge area slightly so that all points are covered
	# Strictly speaking we only need the nearest point to the boundary, but this covers it.
	cosr<-cos(radius+parameters[i, "resolution"])

	dec<-Mdec[idx]
	ra<-Mra[idx]

	# This checks that the points are within radius+resolution of (RA, DEC) by computing spherical distance
	idx0<-idx[(cos(DEC)*cos(dec)*cos(RA-ra)+sin(DEC)*sin(dec)) >= cosr]

	# You can print debugging statement from within this function - useful if modifying condition
	#cat(i, RA, DEC, cosr, sum(( (cos(DEC)*cos(dec)*cos(RA-ra)+sin(DEC)*sin(dec)) >= cosr)), sum(Mi[idx]==i), sum(idx0), "\n")

	return(data.frame(idx=i, band_start=parameters[i, "band_start"], ul=max(Mul[idx0]), snr=max(Msnr[idx0]), npoints=sum(idx0)))
	}

# Iterate over indices
# For this atlas the indices describe pairs (frequency_band, one of twenty sky regions)
# Thus the output list will have 20*number_of_frequency_bands entries
L<-mvl_index_lapply(Matlas$skymaps_extent_index_idx, parameters[,"idx",drop=FALSE], process_band)

cat("\r100 %              \n")

all_data<-do.call(rbind, L)

# Aggregate results so we have one number per frequency band
results<-aggregate(all_data[,c("ul", "snr")], all_data[,"band_start", drop=FALSE], max)

write.table(results, "full_scan_example4_results.csv", col.names=TRUE, row.names=FALSE, sep="\t")
